/*
	ThumbZoomer by Katja Deutschmann <http://www.katjadeutschmann.de>

	@version		1.3
	@dependencies:	mootools 1.2.4 <http://mootools.net/>

	If you hover over a thumbnail, ThumbZoomer will show a bigger version of it in a layer. 
				
	ThumbZoomer requires the MooTools-Framework. It does not support Internet Explorer < 7. 

	To use ThumbZoomer add mootools.js (if not already there) and thumbzoomer.js to your 
	HTML documents head, e.g.:

		<script type="text/javascript" language="javascript" src="js/mootools.js"></script>		
		<script type="text/javascript" language="javascript" src="js/thumbzoomer.js"></script>

	and add a longdesc-attribute with the value "thumzoomer_" plus the source path 
	to the zoomed version to the thumbs image tag, e.g.:
	
		<img src="mysmallimage.jpg" alt="" longdesc="thumbzoomer_mylargeimage.jpg" />

	ThumbZoomer will automatically append a layer for the zoomed thumb to your HTML documents body.

	Function call: 
	window.addEvent('domready', function(){
		new ThumbZoomer({
			// INSERT PARAMS HERE
		}).run();
	});

	You can change the ids of the ThumbZoomer layer and zoomed image as well as the name and the value prefix 
	of the attribute with the zoomed thumb source path by replacing "// INSERT PARAMS HERE" 
	with the desired (comma separated) params:

	@param zoomedThumbId: 						The id of the image inside the thumbzoomer layer (default: zoomedthumb).
	@param zoomedThumbLayerId: 					The id of the thumbzoomer layer (default: zoomedthumblayer).
	@param zoomedThumbSrcAttributeName: 		The name of the attribute specifying the source of the zoomed version (default: longdesc).
	@param zoomedThumbSrcAttributeValuePrefix: 	A prefix indicating the attributes value is for the thumbzoomer (default: thumbzoomer_).		

	Style the layer with CSS, e.g.:
	
	#zoomedthumblayer {
		background-color: #ffffff;
		border: 1px solid #aaaaaa;
		padding: 7px;
	}
*/
var ThumbZoomer = new Class({
	
	zoomedThumbLayer: null,
	zoomedThumb: null,
	zoomedThumbLayerWidth: 0,
	zoomedThumbLayerHeight: 0,	
	windowWidthMinusZoomedThumbLayerWidth: 0,
	windowHeightMinusZoomedThumbLayerHeight: 0,
	xPos: 0,
	yPos: 0,
	ie: false,
	cancelShow: false,
	
	Implements: [Options],
	
	options: {
		zoomedThumbId: 'zoomedthumb',
		zoomedThumbLayerId: 'zoomedthumblayer',
		zoomedThumbSrcAttributeName: 'longdesc',
		zoomedThumbSrcAttributeValuePrefix: 'thumbzoomer_'		
	},
	
	/*
		Initializes the ThumbZoomer. Therefore sets the options and appends the zoomed thumb layer to the body.
	*/
	initialize: function(options) {
		this.setOptions(options);
		
		if (Browser.Engine.trident) {
			this.ie = true;
		}

		// append zoomed thumb layer to the end of body
		this.zoomedThumb = new Element('img', {id: this.options.zoomedThumbId, alt: '', src: ''});
		this.zoomedThumb.setStyles({
			'width': 'auto',
			'height': 'auto'
		});
		this.zoomedThumbLayer = new Element('div', {id: this.options.zoomedThumbLayerId});
		// position "fixed" for all browsers except IE with HTML 4 doctype to avoid layer scrolling away with page
		if (this.ie && document.all[0].text.indexOf('HTML 4') > 0) {
			this.zoomedThumbLayer.setStyles({
				'display': 'block',
				'position': 'absolute',
				'visibility': 'hidden',			
				'z-index': 9998
			});			
		} 
		else {
			this.zoomedThumbLayer.setStyles({
				'display': 'block',
				'position': 'fixed',
				'visibility': 'hidden',			
				'z-index': 9998
			});
		}
		this.zoomedThumbLayer.grab(this.zoomedThumb);
		$(document.body).grab(this.zoomedThumbLayer);		
	},


	/*
		Sets the source of the zoomed thumb.
	*/
	setZoomedThumbSrc: function(zoomedThumbSrc) {
		this.zoomedThumb.setProperty('src', zoomedThumbSrc);		
	},

	
	/*
		Sets the width/height of the zoomed thumb layer and the window width/height minus the zoomed thumb layer width/height (used to flip the thumb layers position to avoid beeing out of the window).
	*/
	setSizes: function() {
		// get width and height of zoomed thumb layer
		this.zoomedThumbLayerWidth = this.zoomedThumbLayer.getSize().x;
		this.zoomedThumbLayerHeight = this.zoomedThumbLayer.getSize().y;

		// get the window with/height minus the zoomed thumb layers width/height 
		// for flipping the zoomed thumb layers position depending on the window width/height
		this.windowWidthMinusZoomedThumbLayerWidth = window.getSize().x - this.zoomedThumbLayerWidth;
		this.windowHeightMinusZoomedThumbLayerHeight = window.getSize().y - this.zoomedThumbLayerHeight;
	},


	/*
		Sets the position of the zoomed thumb layer and shows it when the zoomed thumb is loaded.
	*/
	setZoomedThumbStartPositionAndShow: function(event) {	
		// only show if it hasn't been cancelled already (by mouseleave)
		if (!this.cancelShow) {
			this.setSizes();
			this.setZoomedThumbLayerPosition(event);
			this.showZoomedThumb();							
		}
	},


	/*
		Sets the position of the zoomed thumb layer. To avoid the zoomed thumb layer being out of the window it will flip if it reaches the windows border.
	*/
	setZoomedThumbLayerPosition: function(event) {
		// set sizes
		this.setSizes();
		// get the x- and y-position of cursor 
		// (+ 15 for distance to cursor - looks better and prevents the cursor from accidentally hover over the zoomed thumb layer which would cause mouseleave-event)
		this.xPos = event.client.x + 15;
		this.yPos = event.client.y + 15;

		// flip position to avoid the zoomed thumb layer being out of the window
		if (this.xPos >= this.windowWidthMinusZoomedThumbLayerWidth) {
			this.xPos-= this.zoomedThumbLayerWidth + 25;
		}
		if (this.yPos >= this.windowHeightMinusZoomedThumbLayerHeight) {
			this.yPos-= this.zoomedThumbLayerHeight + 25;
		}

		// set the position to the zoomed thumb layer
		this.zoomedThumbLayer.setPosition({
			x: this.xPos,
			y: this.yPos
		});		
	},


	/*
		Shows the zoomed thumb layer by setting its visibility to visible.
	*/	
	showZoomedThumb: function() {
		this.zoomedThumbLayer.setStyle('visibility', 'visible');	
	},


	/*
		Hides the zoomed thumb layer by setting its visibility to hidden.
	*/
	hideZoomedThumb: function() {
		this.zoomedThumbLayer.setStyle('visibility', 'hidden');	
	},


	/*
		The executing method: Runs the ThumbZoomer.
	*/
	run: function() {
		// reference to class-this
		var self = this;
		var searchString = 'img[' + self.options.zoomedThumbSrcAttributeName + '^=' + self.options.zoomedThumbSrcAttributeValuePrefix + ']';
		// add mouseenter, mousemove and mouseleave event to every image with the specified attribute name and value prefix
		$$(searchString).each(function(item, index, array) {
			item.addEvents({
				// register eventhandler for thumb that sets the source of the zoomed thumb (= attribute value without prefix), sets the position of the zoomed thumb layer and shows it on mouseenter
				mouseenter: function(event) {
					self.cancelShow = false;
					if (item.getProperty(self.options.zoomedThumbSrcAttributeName) != null) {
						var newZoomedThumbSource = item.getProperty(self.options.zoomedThumbSrcAttributeName).replace(self.options.zoomedThumbSrcAttributeValuePrefix, '');
						// show immediately if IE or thumb is the same as the last thumb
						if (self.ie || newZoomedThumbSource == self.zoomedThumb.getProperty('src')) {
							self.setZoomedThumbSrc(newZoomedThumbSource);
							self.setZoomedThumbStartPositionAndShow(event);	
						}
						// show after image has been loaded if no IE and thumb is not the same as the last thumb
						else {
							self.setZoomedThumbSrc(newZoomedThumbSource);
							self.zoomedThumb.addEvent('load', function(event) {	
								self.setZoomedThumbStartPositionAndShow(event);								
							}.bind(self, event));		
						}					
					}
					
				},
				// register eventhandler for thumb that sets the position of the zoomed thumb layer on mousemove
				mousemove: function(event) {
					window.setTimeout(function() {self.setZoomedThumbLayerPosition(event)}, 50);;
				},
				// register eventhandler for thumb that hides the zoomed thumb layer on mouseout
				mouseleave: function() {
					self.hideZoomedThumb();
					self.cancelShow = true;
				}
			});
		});		
	}	
});
